#!/usr/bin/python

version_number = "1.9.2"
release_date = "Wed Sep 30 2003"
#
#
# :::~ Author: Claudio Juan Tessone <tessonec@imedea.uib.es> (c) 2002-2003
#
# Distributed According to GNU Generic Purpose License (GPL)
# Please visit www.gnu.org
#


from .ParamParser import *
from .Executor import *
from .MultiAgrizer import *
from .Agrizer import *
from .TeXParser import *

from .MeanCalculation import *
from .MatrixPlotter import *
